@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,18 +18,18 @@
 """ Options module """
 
 version_string = """\
-Nuitka V0.3.25
-Copyright (C) 2012 Kay Hayen."""
+Nuitka V0.4.5
+Copyright (C) 2013 Kay Hayen."""
 
 from . import Utils
 
 from optparse import OptionParser, OptionGroup
 
-import os, sys, logging
+import sys, logging
 
-# Indicator if we were called as "nuitka-python" in which case we assume some other
-# defaults and work a bit different with parameters.
-is_nuitka_python = Utils.basename( sys.argv[0] ).lower() == "nuitka-python"
+# Indicator if we were called as "nuitka-python" in which case we assume some
+# other defaults and work a bit different with parameters.
+is_nuitka_python = Utils.basename( sys.argv[0] ).lower().startswith( "nuitka-python" )
 
 def getVersion():
     return version_string.split()[1][1:]
@@ -186,7 +186,7 @@ parser.add_option(
     "--python-version",
     action  = "store",
     dest    = "python_version",
-    choices = ( "2.6", "2.7", "3.2" ),
+    choices = ( "2.6", "2.7", "3.2", "3.3" ),
     default = None,
     help    = """Major version of Python to be used, one of '2.6', '2.7', or '3.2'."""
 )
@@ -289,7 +289,7 @@ production. Defaults to off."""
 )
 
 debug_group.add_option(
-    "--unstriped",
+    "--unstripped", "--no-strip", "--unstriped",
     action  = "store_true",
     dest    = "unstriped",
     default = False,
@@ -318,32 +318,15 @@ Compile the would-be regenerated source file. Allows compiling edited C++ files
 C++ compiler for quick debugging changes to the generated source. Defaults to off."""
 )
 
-def decideExperimental():
-    git_branch_name_filename = os.path.join( os.path.dirname( __file__ ), "..", ".git", "HEAD" )
-
-    if not os.path.exists( git_branch_name_filename ):
-        return False
-
-    branch_name = open( git_branch_name_filename ).read().strip()
-    branch_name = branch_name.split( "/" )[-1]
-
-    if branch_name == "master":
-        return os.path.exists( os.path.join( os.path.dirname( __file__ ), "..", "public-repo" ) )
-    elif branch_name.startswith( "hotfix/" ):
-        return False
-    else:
-        return True
-
-if decideExperimental():
-    debug_group.add_option(
-        "--experimental",
-        action  = "store_true",
-        dest    = "experimental",
-        default = False,
-        help    = """\
+debug_group.add_option(
+    "--experimental",
+    action  = "store_true",
+    dest    = "experimental",
+    default = False,
+    help    = """\
 Use features declared as 'experimental'. May have no effect if no experimental features
 are present in the code. Defaults to off."""
-    )
+)
 
 parser.add_option_group( debug_group )
 
@@ -385,8 +368,7 @@ tracing_group.add_option(
     "--show-progress",
     action  = "store_true",
     dest    = "show_progress",
-    # TODO: Enable by default for interactive mode.
-    default = False and sys.stdout.isatty(),
+    default = False,
     help    = """Provide progress information and statistics. Defaults to off."""
 )
 
@@ -413,6 +395,35 @@ Specify the allowed number of parallel C++ compiler jobs. Defaults to the system
 CPU count.""",
 )
 
+parser.add_option(
+    "--improved",
+    action  = "store_true",
+    dest    = "improved",
+    default = False,
+    help    = """\
+Allow minor devitations from CPython behaviour, e.g. better tracebacks, which are not
+really incompatible, but different.""",
+)
+
+parser.add_option(
+    "--warn-implicit-exceptions",
+    action  = "store_true",
+    dest    = "warn_implicit_exceptions",
+    default = False,
+    help    = """\
+Given warnings for implicit exceptions detected at compile time.""",
+)
+
+if False:
+    parser.add_option(
+        "--portable",
+        action  = "store_true",
+        dest    = "is_portable_mode",
+        default = False,
+        help    = """\
+Enable portable mode in build.""",
+    )
+
 if is_nuitka_python:
     count = 0
 
@@ -429,22 +440,18 @@ if is_nuitka_python:
 else:
     extra_args = []
 
+
 options, positional_args = parser.parse_args()
 
 if not positional_args:
     parser.print_help()
 
-    sys.exit( "\nError, need positional argument with python module or main program." )
+    sys.exit( """
+Error, need positional argument with python module or main program.""" )
 
 if options.verbose:
     logging.getLogger().setLevel( logging.DEBUG )
 
-if options.recurse_all and not options.executable:
-    sys.exit( "Error, options '--recurse-all' makes no sense without option '--exe'." )
-
-if options.recurse_stdlib and not options.executable:
-    sys.exit( "Error, options '--recurse-stdlib' makes no sense without option '--exe'." )
-
 def shallTraceExecution():
     return options.trace_execution
 
@@ -481,12 +488,48 @@ def shallFollowAllImports():
 def getShallFollowModules():
     return sum( [ x.split( "," ) for x in options.recurse_modules ], [] )
 
+for any_case_module in getShallFollowModules():
+    if any_case_module.startswith( "." ):
+        bad = True
+    else:
+        for char in "/\\:":
+            if  char in any_case_module:
+                bad = True
+                break
+        else:
+            bad = False
+
+    if bad:
+        sys.exit( """
+Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
+any_case_module )
+
 def getShallFollowInNoCase():
     return sum( [ x.split( "," ) for x in options.recurse_not_modules ], [] )
 
+for no_case_module in getShallFollowInNoCase():
+    if no_case_module.startswith( "." ):
+        bad = True
+    else:
+        for char in "/\\:":
+            if  char in no_case_module:
+                bad = True
+                break
+        else:
+            bad = False
+
+    if bad:
+        sys.exit( """
+Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
+no_case_module )
+
+
 def getShallFollowExtra():
     return sum( [ x.split( "," ) for x in options.recurse_extra ], [] )
 
+def shallWarnImplicitRaises():
+    return options.warn_implicit_exceptions
+
 def isDebug():
     return options.debug
 
@@ -536,7 +579,7 @@ def shallDisableConsoleWindow():
     return options.win_disable_console
 
 def isFullCompat():
-    return True
+    return not options.improved
 
 def isShowProgress():
     return options.show_progress
@@ -549,3 +592,8 @@ def getIntendedPythonVersion():
 
 def isExperimental():
     return hasattr( options, "experimental" ) and options.experimental
+
+def isPortableMode():
+    # Temporarily disabled, until it becomes more usable.
+    return False
+    # return options.is_portable_mode
