@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,21 +22,21 @@ result prediction will help to be smarter, but generally these should not be tha
 about performance critical.
 
 """
-from .NodeBases import CPythonExpressionBuiltinSingleArgBase
+from .NodeBases import ExpressionBuiltinSingleArgBase
 
-from nuitka.transform.optimizations import BuiltinOptimization
+from nuitka.optimizations import BuiltinOptimization
 
-class CPythonExpressionBuiltinBin( CPythonExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinBin( ExpressionBuiltinSingleArgBase ):
     kind = "EXPRESSION_BUILTIN_BIN"
 
     builtin_spec = BuiltinOptimization.builtin_bin_spec
 
-class CPythonExpressionBuiltinOct( CPythonExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinOct( ExpressionBuiltinSingleArgBase ):
     kind = "EXPRESSION_BUILTIN_OCT"
 
     builtin_spec = BuiltinOptimization.builtin_oct_spec
 
-class CPythonExpressionBuiltinHex( CPythonExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinHex( ExpressionBuiltinSingleArgBase ):
     kind = "EXPRESSION_BUILTIN_HEX"
 
     builtin_spec = BuiltinOptimization.builtin_hex_spec
