@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,65 +22,92 @@ that should allow some important optimizations.
 """
 
 from .NodeBases import (
-    CPythonExpressionBuiltinSingleArgBase,
-    CPythonExpressionSpecBasedComputationMixin,
-    CPythonChildrenHaving,
-    CPythonNodeBase
+    ExpressionSpecBasedComputationMixin,
+    ExpressionBuiltinSingleArgBase,
+    ChildrenHavingMixin,
+    NodeBase
 )
 
-from .NodeMakingHelpers import (
-    makeConstantReplacementNode,
-    wrapExpressionWithSideEffects
-)
-
-from nuitka.transform.optimizations import BuiltinOptimization
+from nuitka.optimizations import BuiltinOptimization
 
 from nuitka.Utils import python_version
 
-class CPythonExpressionBuiltinTypeBase( CPythonExpressionBuiltinSingleArgBase ):
+class ExpressionBuiltinTypeBase( ExpressionBuiltinSingleArgBase ):
     pass
 
 
-class CPythonExpressionBuiltinTuple( CPythonExpressionBuiltinTypeBase ):
+class ExpressionBuiltinTuple( ExpressionBuiltinTypeBase ):
     kind = "EXPRESSION_BUILTIN_TUPLE"
 
     builtin_spec = BuiltinOptimization.builtin_tuple_spec
 
 
-class CPythonExpressionBuiltinList( CPythonExpressionBuiltinTypeBase ):
+class ExpressionBuiltinList( ExpressionBuiltinTypeBase ):
     kind = "EXPRESSION_BUILTIN_LIST"
 
     builtin_spec = BuiltinOptimization.builtin_list_spec
 
 
-class CPythonExpressionBuiltinFloat( CPythonExpressionBuiltinTypeBase ):
+class ExpressionBuiltinFloat( ExpressionBuiltinTypeBase ):
     kind = "EXPRESSION_BUILTIN_FLOAT"
 
     builtin_spec = BuiltinOptimization.builtin_float_spec
 
 
-class CPythonExpressionBuiltinBool( CPythonExpressionBuiltinTypeBase ):
+class ExpressionBuiltinBool( ExpressionBuiltinTypeBase ):
     kind = "EXPRESSION_BUILTIN_BOOL"
 
     builtin_spec = BuiltinOptimization.builtin_bool_spec
 
+    def mayProvideReference( self ):
+        # Dedicated code returns "True" or "False" only, which requires no reference
+        return False
+
+    def computeExpression( self, constraint_collection ):
+        value = self.getValue()
+
+        if value is not None:
+            truth_value = self.getValue().getTruthValue()
+
+            if truth_value is not None:
+                from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects, makeConstantReplacementNode
+
+                result = wrapExpressionWithNodeSideEffects(
+                    new_node = makeConstantReplacementNode(
+                        constant = truth_value,
+                        node     = self,
+                    ),
+                    old_node = self.getValue()
+                )
+
+                return result, "new_constant", "Predicted truth value of builtin bool argument"
+
+        return ExpressionBuiltinTypeBase.computeExpression( self, constraint_collection )
 
 
-
-class CPythonExpressionBuiltinIntLongBase( CPythonChildrenHaving, CPythonNodeBase, \
-                                           CPythonExpressionSpecBasedComputationMixin ):
+class ExpressionBuiltinIntLongBase( ChildrenHavingMixin, NodeBase,
+                                    ExpressionSpecBasedComputationMixin ):
     named_children = ( "value", "base" )
 
-    def __init__( self, value, base, source_ref ):
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
+    try:
+        int( base = 2 )
+    except TypeError:
+        base_only_value = False
+    else:
+        base_only_value = True
 
-        if value is None:
+    def __init__( self, value, base, source_ref ):
+        from .NodeMakingHelpers import makeConstantReplacementNode
+
+        NodeBase.__init__( self, source_ref = source_ref )
+
+        if value is None and self.base_only_value:
             value = makeConstantReplacementNode(
                 constant = "0",
                 node     = self
             )
 
-        CPythonChildrenHaving.__init__(
+        ChildrenHavingMixin.__init__(
             self,
             values = {
                 "value" : value,
@@ -88,18 +115,27 @@ class CPythonExpressionBuiltinIntLongBase( CPythonChildrenHaving, CPythonNodeBas
             }
         )
 
-    getValue = CPythonChildrenHaving.childGetter( "value" )
-    getBase = CPythonChildrenHaving.childGetter( "base" )
+    getValue = ChildrenHavingMixin.childGetter( "value" )
+    getBase = ChildrenHavingMixin.childGetter( "base" )
+
+    def computeExpression( self, constraint_collection ):
+        # Children can tell all we need to know, pylint: disable=W0613
 
-    def computeNode( self, constraint_collection ):
         value = self.getValue()
         base = self.getBase()
 
         given_values = []
 
         if value is None:
-            # Note: Prevented that case above.
-            assert base is not None
+            if base is not None:
+                if not self.base_only_value:
+                    from .NodeMakingHelpers import getComputationResult
+
+                    return getComputationResult(
+                        node        = self,
+                        computation = lambda : int( base = 2 ),
+                        description = "int builtin call with only base argument"
+                    )
 
             given_values = ()
         elif base is None:
@@ -110,19 +146,20 @@ class CPythonExpressionBuiltinIntLongBase( CPythonChildrenHaving, CPythonNodeBas
         return self.computeBuiltinSpec( given_values )
 
 
-class CPythonExpressionBuiltinInt( CPythonExpressionBuiltinIntLongBase ):
+class ExpressionBuiltinInt( ExpressionBuiltinIntLongBase ):
     kind = "EXPRESSION_BUILTIN_INT"
 
     builtin_spec = BuiltinOptimization.builtin_int_spec
 
-class CPythonExpressionBuiltinUnicodeBase( CPythonChildrenHaving, CPythonNodeBase, \
-                                           CPythonExpressionSpecBasedComputationMixin ):
+
+class ExpressionBuiltinUnicodeBase( ChildrenHavingMixin, NodeBase,
+                                    ExpressionSpecBasedComputationMixin ):
     named_children = ( "value", "encoding", "errors" )
 
     def __init__( self, value, encoding, errors, source_ref ):
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__( self, source_ref = source_ref )
 
-        CPythonChildrenHaving.__init__(
+        ChildrenHavingMixin.__init__(
             self,
             values = {
                 "value"    : value,
@@ -131,11 +168,13 @@ class CPythonExpressionBuiltinUnicodeBase( CPythonChildrenHaving, CPythonNodeBas
             }
         )
 
-    getValue = CPythonChildrenHaving.childGetter( "value" )
-    getEncoding = CPythonChildrenHaving.childGetter( "encoding" )
-    getErrors = CPythonChildrenHaving.childGetter( "errors" )
+    getValue = ChildrenHavingMixin.childGetter( "value" )
+    getEncoding = ChildrenHavingMixin.childGetter( "encoding" )
+    getErrors = ChildrenHavingMixin.childGetter( "errors" )
+
+    def computeExpression( self, constraint_collection ):
+        # Children can tell all we need to know, pylint: disable=W0613
 
-    def computeNode( self, constraint_collection ):
         args = [
             self.getValue(),
             self.getEncoding(),
@@ -149,13 +188,13 @@ class CPythonExpressionBuiltinUnicodeBase( CPythonChildrenHaving, CPythonNodeBas
 
 
 if python_version < 300:
-    class CPythonExpressionBuiltinStr( CPythonExpressionBuiltinTypeBase ):
+    class ExpressionBuiltinStr( ExpressionBuiltinTypeBase ):
         kind = "EXPRESSION_BUILTIN_STR"
 
         builtin_spec = BuiltinOptimization.builtin_str_spec
 
-        def computeNode( self, constraint_collection ):
-            new_node, change_tags, change_desc = CPythonExpressionBuiltinTypeBase.computeNode(
+        def computeExpression( self, constraint_collection ):
+            new_node, change_tags, change_desc = ExpressionBuiltinTypeBase.computeExpression(
                 self,
                 constraint_collection
             )
@@ -164,7 +203,9 @@ if python_version < 300:
                 str_value = self.getValue().getStrValue()
 
                 if str_value is not None:
-                    new_node = wrapExpressionWithSideEffects(
+                    from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
+
+                    new_node = wrapExpressionWithNodeSideEffects(
                         new_node = str_value,
                         old_node = self.getValue()
                     )
@@ -175,18 +216,18 @@ if python_version < 300:
             return new_node, change_tags, change_desc
 
 
-    class CPythonExpressionBuiltinLong( CPythonExpressionBuiltinIntLongBase ):
+    class ExpressionBuiltinLong( ExpressionBuiltinIntLongBase ):
         kind = "EXPRESSION_BUILTIN_LONG"
 
         builtin_spec = BuiltinOptimization.builtin_long_spec
 
 
-    class CPythonExpressionBuiltinUnicode( CPythonExpressionBuiltinUnicodeBase ):
+    class ExpressionBuiltinUnicode( ExpressionBuiltinUnicodeBase ):
         kind = "EXPRESSION_BUILTIN_UNICODE"
 
         builtin_spec = BuiltinOptimization.builtin_unicode_spec
 else:
-    class CPythonExpressionBuiltinStr( CPythonExpressionBuiltinUnicodeBase ):
+    class ExpressionBuiltinStr( ExpressionBuiltinUnicodeBase ):
         kind = "EXPRESSION_BUILTIN_STR"
 
         builtin_spec = BuiltinOptimization.builtin_str_spec
