@@ -77,10 +77,11 @@ class NodeCheckMetaClass( type ):
 
         type.__init__( mcs, name, bases, dictionary )
 
-# For every node type, there is a test, and then some more members, pylint: disable=R0904
+# For every node type, there is a test, and then some more members,
+# pylint: disable=R0904
 
-# For Python2/3 compatible source, we create a base class that has the metaclass used and
-# doesn't require making a choice.
+# For Python2/3 compatible source, we create a base class that has the metaclass
+# used and doesn't require making a choice.
 NodeMetaClassBase = NodeCheckMetaClass( "NodeMetaClassBase", (object, ), {} )
 
 class NodeBase( NodeMetaClassBase ):
@@ -102,7 +103,8 @@ class NodeBase( NodeMetaClassBase ):
         return True
 
     def __repr__( self ):
-        # This is to avoid crashes, because of bugs in detail. pylint: disable=W0702
+        # This is to avoid crashes, because of bugs in detail.
+        # pylint: disable=W0702
         try:
             detail = self.getDetail()
         except:
@@ -114,7 +116,8 @@ class NodeBase( NodeMetaClassBase ):
             return "<Node %s %s>" % ( self.getDescription(), detail )
 
     def getDescription( self ):
-        """ Description of the node, intented for use in __repr__ and graphical display.
+        """ Description of the node, intented for use in __repr__ and
+            graphical display.
 
         """
         return "%s at %s" % ( self.kind, self.source_ref.getAsString() )
@@ -127,7 +130,8 @@ class NodeBase( NodeMetaClassBase ):
         return {}
 
     def getDetail( self ):
-        """ Details of the node, intended for use in __repr__ and graphical display.
+        """ Details of the node, intended for use in __repr__ and graphical
+            display.
 
         """
         # Virtual method, pylint: disable=R0201
@@ -163,7 +167,8 @@ class NodeBase( NodeMetaClassBase ):
 
         while not parent.isPythonModule():
             if hasattr( parent, "provider" ):
-                # After we checked, we can use it, will be much faster, pylint: disable=E1101
+                # After we checked, we can use it, will be much faster,
+                # pylint: disable=E1101
                 parent = parent.provider
             else:
                 parent = parent.getParent()
@@ -284,13 +289,16 @@ class NodeBase( NodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201,W0613
         return None
 
-    def mayHaveSideEffects( self, constraint_collection ):
+    def mayHaveSideEffects( self ):
         """ Unless we are told otherwise, everything may have a side effect. """
         # Virtual method, pylint: disable=R0201,W0613
 
         return True
 
-    def mayHaveSideEffectsBool( self, constraint_collection ):
+    def isOrderRelevant( self ):
+        return self.mayHaveSideEffects()
+
+    def mayHaveSideEffectsBool( self ):
         """ Unless we are told otherwise, everything may have a side effect. """
         # Virtual method, pylint: disable=R0201,W0613
 
@@ -337,7 +345,7 @@ class NodeBase( NodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def getIntegerValue( self, constraint_collection ):
+    def getIntegerValue( self ):
         """ Node as integer value, if possible."""
         # Virtual method, pylint: disable=R0201,W0613
         return None
@@ -695,20 +703,17 @@ class ClosureTakerMixin:
         # There is no maybe with closures. It means, it is closure variable in
         # this case.
         if result.isMaybeLocalVariable():
-            # This mixin is used with nodes only, but doesn't want to inherit from
-            # it, pylint: disable=E1101
+            # This mixin is used with nodes only, but doesn't want to inherit
+            # from it, pylint: disable=E1101
             result = self.getParentModule().getVariableForClosure(
                 variable_name = variable_name
             )
 
         return self.addClosureVariable( result )
 
-    def addClosureVariable( self, variable, global_statement = False ):
+    def addClosureVariable( self, variable ):
         variable = variable.makeReference( self )
 
-        if variable.isModuleVariable() and global_statement:
-            variable.markFromGlobalStatement()
-
         self.taken.add( variable )
 
         return variable
@@ -736,11 +741,13 @@ class ClosureTakerMixin:
             return None
 
     def isEarlyClosure( self ):
-        """ Normally it's good to lookup name references immediately, but not for functions.
+        """ Early closure taking means immediate binding of references.
 
-        in case of a function body it is not allowed to do that, because a later
-        assignment needs to be queried first. Nodes need to indicate via this if they
-        would like to resolve references at the same time as assignments.
+        Normally it's good to lookup name references immediately, but not for
+        functions. In case of a function body it is not allowed to do that,
+        because a later assignment needs to be queried first. Nodes need to
+        indicate via this if they would like to resolve references at the same
+        time as assignments.
         """
 
         return self.early_closure
@@ -753,19 +760,19 @@ class ExpressionMixin:
     def isCompileTimeConstant( self ):
         """ Has a value that we can use at compile time.
 
-            Yes or no. If it has such a value, simulations can be applied at compile time
-            and e.g. operations or conditions, or even calls may be executed against it.
+            Yes or no. If it has such a value, simulations can be applied at
+            compile time and e.g. operations or conditions, or even calls may
+            be executed against it.
         """
         # Virtual method, pylint: disable=R0201
-
         return False
 
     def getCompileTimeConstant( self ):
-        assert self.isCompileTimeConstant(), ( self, "asked for compile time constant" )
+        assert self.isCompileTimeConstant(), self
 
         assert False
 
-    def getTruthValue( self, constraint_collection ):
+    def getTruthValue( self ):
         """ Return known truth value. The "None" value indicates unknown. """
 
         if self.isCompileTimeConstant():
@@ -776,22 +783,22 @@ class ExpressionMixin:
     def isKnownToBeIterable( self, count ):
         """ Can be iterated at all (count is None) or exactly count times.
 
-            Yes or no. If it can be iterated a known number of times, it may be asked to
-            unpack itself.
+            Yes or no. If it can be iterated a known number of times, it may
+            be asked to unpack itself.
         """
 
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def isKnownToBeIterableAtMin( self, count, constraint_collection ):
+    def isKnownToBeIterableAtMin( self, count ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def isKnownToBeIterableAtMax( self, count, constraint_collection ):
+    def isKnownToBeIterableAtMax( self, count ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def getIterationNext( self, constraint_collection ):
+    def getIterationNext( self ):
         # Virtual method, pylint: disable=R0201,W0613
         return None
 
@@ -804,30 +811,33 @@ class ExpressionMixin:
         # Virtual method, pylint: disable=R0201
         return True
 
-    def getIterationLength( self, constraint_collection ):
+    def getIterationLength( self ):
         """ Value that "len" or "PyObject_Size" would give, if known.
 
-        Otherwise it is "None" to indicate unknown.
+            Otherwise it is "None" to indicate unknown.
         """
 
         # Virtual method, pylint: disable=R0201
         return None
 
-    def getStringValue( self, constraint_collection ):
+    def getStringValue( self ):
         """ Node as integer value, if possible."""
         # Virtual method, pylint: disable=R0201,W0613
         return None
 
-    def getStrValue( self, constraint_collection ):
+    def getStrValue( self ):
         """ Value that "str" or "PyObject_Str" would give, if known.
 
-        Otherwise it is "None" to indicate unknown.
+            Otherwise it is "None" to indicate unknown.
         """
-        string_value = self.getStringValue( constraint_collection )
+        string_value = self.getStringValue()
 
         if string_value is not None:
             from .NodeMakingHelpers import makeConstantReplacementNode
 
+            # TODO: Side effects should be considered, getStringValue may be
+            # omitting effects.
+
             return makeConstantReplacementNode(
                 node     = self,
                 constant = string_value
@@ -839,15 +849,33 @@ class ExpressionMixin:
         # print "onRelease", self
         pass
 
+    def computeExpressionRaw( self, constraint_collection ):
+        """ Compute an expression.
+
+            Default behavior is to just visit the child expressions first, and
+            then the node "computeExpression". For a few cases this needs to
+            be overloaded, e.g. conditional expressions.
+        """
+
+        # First apply the sub-expressions, as they are evaluated before.
+        sub_expressions = self.getVisitableNodes()
+
+        for sub_expression in sub_expressions:
+            constraint_collection.onExpression( sub_expression )
+
+        # Then ask ourselves to work on it.
+        return self.computeExpression( constraint_collection )
+
     def computeExpressionAttribute( self, lookup_node, attribute_name, constraint_collection ):
-        # By default, an attribute lookup may change everything about the lookup source.
-        # Virtual method, pylint: disable=R0201,W0613
+        # By default, an attribute lookup may change everything about the lookup
+        # source. Virtual method, pylint: disable=R0201,W0613
         constraint_collection.removeKnowledge( lookup_node )
 
         return lookup_node, None, None
 
     def computeExpressionSubscript( self, lookup_node, subscript, constraint_collection ):
-        # By default, an subscript may change everything about the lookup source.
+        # By default, an subscript may change everything about the lookup
+        # source.
         constraint_collection.removeKnowledge( lookup_node )
 
         return lookup_node, None, None
@@ -871,6 +899,7 @@ class ExpressionMixin:
     def onContentEscapes( self, constraint_collection ):
         pass
 
+
 class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def __init__( self ):
         self.computed_attribute = False
@@ -878,17 +907,18 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def isCompileTimeConstant( self ):
         """ Has a value that we can use at compile time.
 
-            Yes or no. If it has such a value, simulations can be applied at compile time
-            and e.g. operations or conditions, or even calls may be executed against it.
+            Yes or no. If it has such a value, simulations can be applied at
+            compile time and e.g. operations or conditions, or even calls may
+            be executed against it.
         """
         # Virtual method, pylint: disable=R0201
 
         return True
 
-    def mayHaveSideEffects( self, constraint_collection ):
+    def mayHaveSideEffects( self ):
         return False
 
-    def mayHaveSideEffectsBool( self, constraint_collection ):
+    def mayHaveSideEffectsBool( self ):
         return False
 
     def computeExpressionOperationNot( self, not_node, constraint_collection ):
@@ -897,7 +927,8 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
         return getComputationResult(
             node        = not_node,
             computation = lambda : not self.getCompileTimeConstant(),
-            description = "Compile time constant negation truth value precomputed."
+            description = """\
+Compile time constant negation truth value precomputed."""
         )
 
 
@@ -914,7 +945,9 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
             return getComputationResult(
                 node        = lookup_node,
                 computation = lambda : getattr( value, attribute_name ),
-                description = "Attribute lookup to %s precomputed." % attribute_name
+                description = "Attribute lookup to %s precomputed." % (
+                    attribute_name
+                )
             )
 
         self.computed_attribute = True
@@ -937,7 +970,8 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def computeExpressionSlice( self, lookup_node, lower, upper, constraint_collection ):
         from .NodeMakingHelpers import getComputationResult
 
-        # TODO: Could be happy with predictable index values and not require constants.
+        # TODO: Could be happy with predictable index values and not require
+        # constants.
         if lower is not None:
             if upper is not None:
                 if lower.isCompileTimeConstant() and upper.isCompileTimeConstant():
@@ -1030,8 +1064,8 @@ class ExpressionBuiltinNoArgBase( NodeBase, ExpressionMixin ):
     def computeExpression( self, constraint_collection ):
         from .NodeMakingHelpers import getComputationResult
 
-        # The lamba is there for make sure that no argument parsing will reach the builtin
-        # function at all, pylint: disable=W0108
+        # The lamba is there for make sure that no argument parsing will reach
+        # the builtin function at all, pylint: disable=W0108
         return getComputationResult(
             node        = self,
             computation = lambda : self.builtin_function(),
@@ -1069,9 +1103,9 @@ class ExpressionBuiltinSingleArgBase( ExpressionChildrenHavingBase,
 
 
 class SideEffectsFromChildrenMixin:
-    def mayHaveSideEffects( self, constraint_collection ):
+    def mayHaveSideEffects( self ):
         for child in self.getVisitableNodes():
-            if child.mayHaveSideEffects( constraint_collection ):
+            if child.mayHaveSideEffects():
                 return True
         else:
             return False
