@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -34,7 +34,7 @@ else:
     active = True
 
 if "PYTHON" not in os.environ:
-    os.environ[ "PYTHON" ] = "python" if os.name != "nt" else sys.executable
+    os.environ[ "PYTHON" ] = sys.executable
 
 def check_output(*popenargs, **kwargs):
     from subprocess import Popen, PIPE, CalledProcessError
@@ -72,21 +72,27 @@ for filename in sorted( os.listdir( "." ) ):
         active = True
 
     if active:
-        if filename not in ( "module_exits", "package_missing_init", "main_raises", "main_raises2" ):
+        expected_errors = [
+            "module_exits", "main_raises", "main_raises2",
+            "package_contains_main"
+        ]
+
+        # Allowed after Python3, packages need no more "__init__.py"
+
+        if python_version < "3.3":
+            expected_errors.append( "package_missing_init" )
+
+        if filename not in expected_errors:
             extra_flags = [ "expect_success" ]
         else:
             extra_flags = [ "expect_failure" ]
 
-        if filename in ( "package_missing_init", "dash_import", ):
+        if filename in ( "package_missing_init", "dash_import", "reimport_main" ):
             extra_flags.append( "ignore_stderr" )
 
         extra_flags.append( "remove_output" )
 
-        os.environ[ "PYTHONPATH" ] = os.path.abspath( filename )
-
-        if filename == "syntax_errors":
-            os.environ[ "NUITKA_EXTRA_OPTIONS" ] = "--recurse-all --execute-with-pythonpath"
-        elif filename == "plugin_import":
+        if filename == "plugin_import":
             os.environ[ "NUITKA_EXTRA_OPTIONS" ] = "--recurse-all --recurse-directory=%s/some_package" % filename
         else:
             os.environ[ "NUITKA_EXTRA_OPTIONS" ] = "--recurse-all"
